// This script plots a Trochoid or Cycloid on the x axis.
// A common Trochoid has the bRadius equal to the aRadius, and is a Cycloid.
// Interpcurve factory is used, to avoid cusp rounding.
// Rewritten by Brian McMillin, September 2018.
// A Brachistochrone is an inverted Cycloid.

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function EvalCycloid( t, aRadius, bRadius, invert )
{
	var phi = t * Math.PI * 2.0;
	var x = aRadius * phi - bRadius * Math.sin( phi );
	var y = invert * (aRadius - bRadius * Math.cos( phi ));
	var z = 0.0;
		
	return moi.vectorMath.createPoint( x, y, z );
}

function Update()
{
	var factory = moi.command.createFactory( 'interpcurve' );

	var aRadius = moi.ui.commandUI.aRadius.numericValue;
	var bRadius = moi.ui.commandUI.bRadius.numericValue;
	var numpoints = moi.ui.commandUI.numpoints.integerValue;
	var invertValue = moi.ui.commandUI.invert.value;
	var invert = 1.0; // Do not invert trochoid.
	if ( invertValue )
		invert = -1.0; // Invert trochoid.	
	var ignore_bRadius = moi.ui.commandUI.ignore_bRadius.value;	
	if ( ignore_bRadius )
		bRadius = aRadius; // Plot cycloid, a common trochoid.
	
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);
		
		var pt = EvalCycloid( t, aRadius, bRadius, invert );		
		
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}
	
	factory.update();
	
	return factory;
}

function DoCycloid()
{
	var factory = Update();

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'aRadius' || dlg.event == 'bRadius' || dlg.event == 'numpoints' || dlg.event == 'invert' || dlg.event == 'ignore_bRadius' )
		{
			factory.cancel();
			factory = Update();
		}
	}
	
	factory.commit();
}

DoCycloid();
