#include "GetPointOsnappedOnCurve.js"
#include "GetPoint.h"

var g_CircleFactory = null;

function GetGsnap( pickedpt )
{
	for ( var i = 0; i < pickedpt.numOsnaps; ++i )
	{
		if ( pickedpt.osnap(i).isOnCurve )
			return pickedpt.osnap(i);
	}

	return null;
}

function DoUpdate( pointpicker )
{
	if ( g_CircleFactory )
	{
		g_CircleFactory.cancel();
		g_CircleFactory = null;
	}

	var osnap = GetGsnap( pointpicker.pt );
	if ( !osnap )
		return;

	var t = osnap.parameter;
	var crv = osnap.object;

	var pt = crv.evaluatePoint( t );
	var tangent = crv.evaluateTangent( t );
	var curvature = crv.evaluateCurvature( t );

	var magsq = (curvature.x * curvature.x) + (curvature.y * curvature.y) + (curvature.z * curvature.z);
	if ( magsq > 0.000001 )
	{
		var mag = Math.sqrt(magsq);

		var radius = 1.0 / mag;
		curvature.scale( radius );

		pt.x += curvature.x * radius;
		pt.y += curvature.y * radius;
		pt.z += curvature.z * radius;

		var frame = moi.vectorMath.createFrame( pt, tangent, curvature );
		
		g_CircleFactory = moi.command.createFactory( 'circle' );
		g_CircleFactory.setInput( 0, true ); // "Is Radius"
		g_CircleFactory.setInput( 1, frame );
		g_CircleFactory.setInput( 3, radius );
		g_CircleFactory.update();
	}
}

function DoDimCurvature()
{
	var ui = moi.ui;
	var pointpicker = ui.createPointPicker();
	pointpicker.enableOnObjectSnap = true;

	pointpicker.bindFuncPostUpdate( DoUpdate );
	if ( !GetPointOsnappedOnCurve( pointpicker ) )
		return;

	if ( !g_CircleFactory )
		return;

	var circle = g_CircleFactory.getCreatedObjects().item(0);
	var circlept = pointpicker.pt;

	pointpicker.clearBindings();
	pointpicker.restrictToPlane( circle.planarFrame, true );

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'PointPrompt' );
	moi.ui.showUI( 'ArrowPrompt' );
	moi.ui.showUI( 'DimOptions' );
	moi.ui.endUIUpdate();

	var factory = moi.command.createFactory( 'dimradius' );
	factory.setInput( 0, circle );
	moi.ui.bindUIToInput( 'type', 'value', factory.getInput(1) );
	moi.ui.bindUIToInput( 'AnnotationPreset', 'value', factory.getInput(4) );

	pointpicker.bindResultPt( factory.getInput(2) );

	circle.setHitTest( false );
	var res = GetPoint( pointpicker );
	circle.setHitTest( true );

    if ( !res )
		return;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'ArrowPrompt' );
	moi.ui.showUI( 'TextPrompt' );
	moi.ui.endUIUpdate();

	pointpicker.clearBindings();
	pointpicker.bindResultPt( factory.getInput(3) );
	if ( !GetPoint( pointpicker ) )
		return;

	factory.commit();
	g_CircleFactory.cancel();	
}

DoDimCurvature();
