// config: norepeat

#include "GetObjects.js"

function WrapWithObjectList( obj )
{
	var list = moi.geometryDatabase.createObjectList();
	list.addObject( obj );
	return list;
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function Update( objects, factories )
{
	Cancel( factories );

	// dir is a string value one of 'x', 'y', 'z'
	var dir = moi.ui.commandUI.dir.value;
	
	// Sort the objects by their bounding box center points.

	var sorted = [];
	
	for ( var i = 0; i < objects.length; ++i )
	{
		var obj = objects.item(i);
		var bbox = obj.getBoundingBox( true ); // true = Use high accuracy bounds
		var entry = { object:obj, bounds:bbox, sortval:bbox.center[dir] };
		sorted.push( entry );
	}

	sorted.sort( function(a,b){ return a.sortval - b.sortval } );
	

	var use_length = moi.ui.commandUI.use_length.value;
	var len = 0.0;
	if ( use_length )
		len = moi.ui.commandUI.len.value;


	var total_space = sorted[sorted.length-1].bounds.min[dir] - sorted[0].bounds.max[dir];

	if ( use_length && Math.abs(len) > 0.0 )
	{
		total_space = len;
		total_space -= sorted[0].bounds.max[dir] - sorted[0].bounds.min[dir];
		total_space -= sorted[sorted.length-1].bounds.max[dir] - sorted[sorted.length-1].bounds.min[dir];
	}
		

	var occupied_space = 0;
	for ( var i = 1; i < sorted.length-1; ++i )
		occupied_space += sorted[i].bounds.max[dir] - sorted[i].bounds.min[dir];
		
	var available_space = total_space - occupied_space;
		
	var target_space_between = available_space / (sorted.length-1);

	
	var prev_end = sorted[0].bounds.max[dir];
	
	for ( var i = 1; i < sorted.length; ++i )
	{
		if ( i == sorted.length - 1 && !use_length )
			break;

		var current_dist_between = sorted[i].bounds.min[dir] - prev_end;
		var delta = target_space_between - current_dist_between;
		
		var pt = moi.vectorMath.createPoint(0,0,0);
		pt[dir] = delta;
		
		var factory = moi.command.createFactory( 'move' );
		factory.setInput( 0, WrapWithObjectList(sorted[i].object) );
		factory.setInput( 1, moi.vectorMath.createPoint(0,0,0) );
		factory.setInput( 2, pt );
		factory.update();
		factories.push( factory );
		
		prev_end = sorted[i].bounds.max[dir] + delta;
	}
}

function DoDistribute()
{
	var objectpicker = moi.ui.createObjectPicker();
	if ( !GetObjects( objectpicker ) )
		return;
		
	var objects = objectpicker.objects;
	if ( objects.length < 2 )
	{
		moi.ui.alert( 'Error - requires 2 or more objects selected.' );
		return;
	}
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'DistributeOptions' );
	moi.ui.endUIUpdate();
	
	var factories = [];
	Update( objects, factories );	
	
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
			return;
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'dir' || dlg.event == 'use_length' || dlg.event == 'len' )
			Update( objects, factories );
	}
	
	Commit( factories );
}

DoDistribute();
