function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//	Define cosh, sinh and tanh.
function cosh(x)
{
	return (Math.exp(x) + Math.exp(-x))/2.0;
}

function sinh(x)
{
	return (Math.exp(x) - Math.exp(-x))/2.0;
}

function tanh(x)
{
	return (Math.exp(x) - Math.exp(-x))/(Math.exp(x) + Math.exp(-x));
}

// Jacobi amplitude function Jacobi_am(u,m) 
// phi = am(u,k) = am(u|m) = am(u\alpha) where Parameter m = k*k, k = sin(alpha).
// Previous version was modified with Julia code: 
// and reviewed c code from here: http://www.mymathlib.com/functions/

//  ORIGINAL c-code Description: (VERY GOOD)                                  
//     Let F(phi,k) = F(phi \ alpha) = F(phi | m) be Legendre's elliptic      
//     function of the first kind with modulus k, modular angle alpha where   
//     k = sin(alpha) or parameter m where m = k^2, i.e.                      
//        F(phi,k) = Integral(0,phi) dtheta / sqrt(1 - k^2 sin^2(theta))      
//        F(phi \ alpha) = Integral(0,phi) dtheta /                           
//                                        sqrt(1 - sin^2(alpha) sin^2(theta)) 
//        F(phi | m) = Integral(0,phi) dtheta / sqrt(1 - m sin^2(theta))      
//                                                                            
//     This Jacobi elliptic amplitude function, am, is defined as             
//               am(u,k) = am(u \ alpha) = am(u | m)  = phi                   
//     where u = F(phi,k) = F(phi \ alpha) = F(phi | m).                      
//                                                                            
//     The common mean method, sometimes called the Gauss transform method,   
//     is a variant of the descending Landen transformation in which two      
//     sequences are formed: Setting a[0] = 1 and g[0] = 1-m, a[i] is the     
//     arithmetic average and g[i] is the geometric mean of a[i-1] and g[i-1],
//     i.e. a[i+1] = (a[i] + g[i])/2 and g[i+1] = sqrt(a[i]*g[i]).  The       
//     sequences, a[i] and g[i], satisfy the inequalities                     
//     g[0] < g[1] < ... < a[1] < a[0].  Further, lim g[n] = lim a[n].        
//                                                                            
//     Set phi[n] = 2^n a[n] u, the recursively compute phi[n-1] by           
//        phi[n-1] = [ phi[n] + arcsin( c[n] sin(phi[n]) / a[n] ] / 2         
//     for until n = 1.  Then am(u,k) = am(u \ alpha) = am(u | m) = phi[0].   
//                                                                            
//  Return Value:                                                             
//     The amplitude am(u,m) in radians.                                      

// Uses: Math.sqrt(), Math.abs(), Math.asin(), Math.atan(), Math.sin(), Math.PI

function Jacobi_am(uValue,m)
{
	var asav = new Array();
	var csav = new Array();	
	var k = Math.sqrt(m);
	var m1 = 1.0 - m;	
	var phi;
	var PI_2 = Math.PI/2;	
	var tol = 10.0e-11;
 	
    if (uValue == 0.0) return 0.0;

	if (m < tol) //A&S 16.13.4
	{
		var result = uValue - 0.25*m*(uValue - 0.5 * Math.sin(2.0*uValue));
		return result;
	}	

    if (m1 < tol) //A&S 16.15.4
	{
        var t = tanh(uValue);
        result = Math.asin(t) + 0.25 * m1 * (t - uValue * (1. - t*t)) * cosh(uValue);	
		return result;		
	}
	
	// Check special case k = 1
	if ( k == 1.0 ) return 2.0 * Math.atan( Math.exp(uValue) ) - PI_2;

	// If k > 1, then perform a Jacobi modulus transformation. NOT DONE

	asav[0] = 1.0;
	b = Math.sqrt(m1);
	csav[0] = k; 
	var n2 = 1.0;
	var i = 0;

	while( Math.abs( (csav[i]/asav[i]) ) > tol )
	{
		if( i > 10 )
		{
		/*mtherr( "ellpjf", OVERFLOW );*/
		break;
		}
		var a = asav[i];
		++i;
		csav[i] = 0.5 * ( a - b );
		var t = Math.sqrt( a * b );
		asav[i] = 0.5 * ( a + b );
		b = t;
		n2 += n2;
	}
	/* backward recurrence */
	// Find phi_subzero, starting at a[n] and c[n], and decrementing to n=0.
	// See paragraph 22.20.2 and .3 and .4 of nist website.
	var phi = n2 * asav[i] * uValue;
	do
	{
		t = csav[i] * Math.sin(phi) / asav[i];
		b = phi;
		phi = 0.5 * (Math.asin(t) + phi);
	}
	while( --i );
	return phi;
}

//  Computes the R_F from Carlson symmetric form 
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
function RF( X, Y, Z )
{
	//var X, Y, Z;
	var result, A, lamda, dx, dy, dz;
	var MinError = 0.0000001;

	do
	{
       lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);

       X = (X + lamda) / 4;
       Y = (Y + lamda) / 4;
       Z = (Z + lamda) / 4;

       A = (X + Y + Z) / 3;

       dx = (1 - X / A);
       dy = (1 - Y / A);
       dz = (1 - Z / A);
	}
    while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

    var E2, E3;
    E2 = dx * dy + dy * dz + dz * dx;
    E3 = dy * dx * dz;

    //http://dlmf.nist.gov/19.36#E1
    result = 1 - (1 / 10) * E2 + (1 / 14) * E3 + (1 / 24) * E2 * E2 - (3 / 44) * E2 * E3 - (5 / 208) * E2 * E2 * E2  + (3 / 104) * E3 * E3 + (1 / 16) * E2 * E2 * E3;

    result *= (1 / Math.sqrt(A));
    return result;
}

//  Computes the R_D from Carlson symmetric form
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
//  Constructed from R_J(x,y,z,z) which is equal to R_D(x,y,z)

function RD( X, Y, Z )
{
	//var X, Y, Z;
    var sum, fac, lamda, dx, dy, dz, A;
    var MinError = 0.000000001;

    sum = 0;
    fac = 1;

    do
{
		lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);
        sum += fac / (Math.sqrt(Z) * (Z + lamda));

        fac /= 4;

        X = (X + lamda) / 4;
        Y = (Y + lamda) / 4;
        Z = (Z + lamda) / 4;

        A = (X + Y + 3 * Z) / 5;

        dx = (1 - X / A);
        dy = (1 - Y / A);
        dz = (1 - Z / A);
	}
    while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

    var E2, E3, E4, E5;
    E2 = dx * dy + dy * dz + 3 * dz * dz + 2 * dz * dx + dx * dz + 2 * dy * dz;
    E3 = dz * dz * dz + dx * dz * dz + 3 * dx * dy * dz + 2 * dy * dz * dz + dy * dz * dz + 2 * dx * dz * dz;
    E4 = dy * dz * dz * dz + dx * dz * dz * dz + dx * dy * dz * dz + 2 * dx * dy * dz * dz;
    E5 = dx * dy * dz * dz * dz;

    //http://dlmf.nist.gov/19.36#E2
    var result = (1 - (3 / 14) * E2 + (1 / 6) * E3 + (9 / 88) * E2 * E2 - (3 / 22) * E4 - (9 / 52) * E2 * E3 + (3 / 26) * E5 - (1 / 16) * E2 * E2 * E2 + (3 / 40) * E3 * E3 + (3 / 20) * E2 * E4 + (45 / 272) * E2 * E2 * E3 - (9 / 68) * (E3 * E4 + E2 * E5));

    result = 3.0 * sum + fac * result / (A * Math.sqrt(A));
    return result;
}

// Returns the incomplete elliptic integral of the first kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei1F(angle, kk)
{
	var u = angle;
	var cosu2 = Math.cos(u);
	cosu2 *= cosu2;
	var sinu2 = Math.sin(u);
	sinu2 *= sinu2;	
	var result = Math.sin(u) * RF(cosu2, (1 - kk * sinu2), 1);
    return result;
}

// Returns the incomplete elliptic integral of the second kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei2E(angle, kk)
{ 
	var u = angle;
	var result = Math.sin(u) * RF(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1) - (1 / 3) * kk * Math.pow(Math.sin(u),3) * RD(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1);
	return result;
}

function Update(s_qtr, uValue)
{
	var curvefactory = moi.command.createFactory( 'interpcurve' );
	
	var UI = moi.ui.commandUI;
	var numpoints = UI.numpoints.integerValue;

	// Calculate values that do not vary with sValue.	
	var aValue = Math.sqrt((1-uValue)/2); // = a2Value.
	var PI_2 = Math.PI/2;
	var y = 0.0;
	// Calculate sMax where z = 0. sMax has 2 possible values based on uValue.
	if ( uValue >= 1 )
	{
		alert ("Shape =" + uValue + "\nShape must be < 1" );
		break;
	}
	if ( uValue < -1 )
	{
		var k = Math.sqrt( 2 / ( 1-uValue ) ); //k2
		var m = k * k;
		var Kofm =  ei1F( PI_2, m ); //complete elliptic integral of 1st kind
		var divmult = Math.sqrt( 2 * ( 1-uValue ));
		var sMax = Kofm / divmult;	
	}
	
	else if ( uValue >= -1 || uValue < 1 )
	{
		var k = Math.sqrt(( 1-uValue ) / 2 ); //k1
		var m = k * k;
		var Kofm =  ei1F( PI_2, m );
		var sMax = Kofm / 2;
	}		
	
	for ( var j = 0; j < numpoints; ++j )
	{
		var sval = j / (numpoints - 1);
		var sValue = sval * sMax;
		
		if ( uValue < -1 ) var sArg = sValue * divmult;
		
		else if ( uValue > -1 || uValue < 1 ) var sArg = sValue * 2;
			
		// With k = k1, get amValue(sArg,k) and cnValue(sArg,k), if (-1 < u < 1).
		// With k = k2, get amValue(sArg,k) and dnValue(sArg,k), if (u = -1).
		// m is the square of k.

		amValue = Jacobi_am(sArg,m);// phi
		cnValue = Math.cos(amValue);
		snValue = Math.sin(amValue);			
		dnValue = Math.sqrt( 1.0 - m * snValue * snValue);	
		EValue = ei2E(amValue,m);

		if (uValue < -1)
		{
			var x = aValue * EValue + uValue * sValue;
			var z = aValue * dnValue;	
		}
		else if ( uValue > -1 || uValue < 1 )
		{ 
			var x = EValue - sValue; 
			var z = aValue * cnValue;
		}
		else if ( uValue == -1 )
		{
			x = tanh( 2*sValue ) - sValue;
			z = 1 / sinh( 2*sValue );
		}
		
		 //alert ("x =" + x + "\nz =" + z + "\namValue =" + amValue + "\nuValue =" + uValue + "\nEValue =" + EValue  + "\ncnValue =" + cnValue );
		var pt = moi.vectorMath.createPoint( x, y, z );		
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );
	}
	curvefactory.update();
	return curvefactory;
}

function DoElastica()
{
	var s_qtr = moi.ui.commandUI.s_qtr.numericValue;
	var uValue = moi.ui.commandUI.uValue.numericValue;
	
// Note that "prefix" moi.ui.commandUI. is needed for code moved from .js to .htm file.
	var curvefactory = Update(s_qtr, uValue);
//	var curvefactory = moi.ui.commandUI.Update(uMax, hValue, eValue); //for code in .htm file.
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			curvefactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 's_qtr' || dlg.event == 'uValue' || dlg.event == 'numpoints' || dlg.event == 'Sslider'  || dlg.event == 'Uslider' || dlg.event == 'bot' || dlg.event == 'slim' )
		{
		
		var s_qtr = moi.ui.commandUI.s_qtr.numericValue;
		var uValue = moi.ui.commandUI.uValue.numericValue;
	
/*
	if (hValue >= 0.0)
	{
		hValue = -0.01;
		alert ( " hValue must be < 0" + "\nhValue reset" ); 
	}

	if (eValue <= 0.5)
		{
			eValue = 0.501;
			alert ( " eValue must be > 0.5" + "\neValue reset" ); 
		}	
*/		
		
			curvefactory.cancel();
			var curvefactory = Update(s_qtr, uValue); //for code in .js
			//curvefactory = moi.ui.commandUI.Update(hValue, eValue); // for code in .htm
			moi.ui.commandUI.updating = false;			
		}
	}
	curvefactory.commit();
}
DoElastica();







