//	FrenetCurveRK2 script.

//	This script uses Runge-Kutta2, midpoint method.
//	See attached pdf "Frenet-Serret Javascript program for Moment of Inspiration."
//	This script uses Runge-Kutta2, midpoint method.
//	DRAFT by Brian McMillin January 23 - February 21, 2012

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//	Michael's MoI bisector script has the needed code for dot product, length, and
//	normalize.  Reference book: Geometric Tools for Computer Graphics.  Section 4.4
//	of the book, page 117, shows the dot product code.  Page 119 shows the cross
//	product code, which is similarly converted to MoI type javascript, using v0
//	instead of the u1 vector used in the book.  Note that v0 and v1 are Vectors,  
//	and v0.x, v0.y, v0.z, v1.x,..., are the x, y, and z parts of the vectors.
//	(crossproduct creates the new "pseudo" vector, which is still a vector.)
//	Scalar formula is on page 115.  Quote from Michael on MoI forum,  "...to build a vector directly from x, y, z values you actually want to use createPoint() to do that, since you will actually hold a point or a vector in the exact same point object since they both just need to contain an x,y,z triplet value. Since both vectors and points have the exact same data in them, there's just one class for them since otherwise they would just have all the exact same properties and methods and everything.  So the return value from crossProduct should be sent back with createPoint() - you only use makeVector for building a vector that represents a displacement between a "from point" and a "to point".  

function crossProduct( v0, v1 )
{
	var x = (v0.y * v1.z) - (v0.z * v1.y);
	var y = (v0.z * v1.x) - (v0.x * v1.z);
	var z = (v0.x * v1.y) - (v0.y * v1.x);
	return moi.vectorMath.createPoint( x, y, z );
}

function scaleVector( v0, factor )
{
	var x = v0.x * factor;
	var y = v0.y * factor;
	var z = v0.z * factor;
	return moi.vectorMath.createPoint( x, y, z );
}	

function addVectors( v0, v1 )
{
	var x = v0.x + v1.x;
	var y = v0.y + v1.y;
	var z = v0.z + v1.z;
	return moi.vectorMath.createPoint( x, y, z );
}
	
function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}
	
function lengthsq( v )
{
	return dotproduct ( v, v );
}

function normalize( v )
{
	var lensq = lengthsq(v);
	if ( lensq < 1.0e-12 )
		return; // degenerate
	var invlen = 1.0 / Math.sqrt(lensq);
//	v.x *= invlen;
//	v.y *= invlen;
//	v.z *= invlen;
	var x = v.x * invlen;
	var y = v.y * invlen;
	var z = v.z * invlen;
	return moi.vectorMath.createPoint( x, y, z );	
	
}

function Update(r_0, d_r, S_0, D_S, L, numPts)
{
	var logfactory = moi.command.createFactory( 'curve' );	
	var vm = moi.vectorMath;   //abbreviation
	var gd = moi.geometryDatabase;  //abbreviation	

//	r_0 = initial radius of curvature. 
//	d_r = delta_r = change in radius of curvature along arc.
//	S_0 = sigma0 = initial radius of torsion.
//	D_S = delta_sigma = change in radius of torsion along arc.
//	L = total arc length.
//	si = arc length at index i.
//	K = K(si) = kappa(si) = curvature = 1/(radius0 + (delta_r * si))
//	tau = tau(si) = torsion = 1/(sigma0 + (delta_sigma * si))
//	|radius0 + delta_r * s| > 0 and | sigma0 + delta_sigma * s | > 0
//	ds is the interval over which to calculate the next value for each of

//  Tangent, Normal, and Binormal (P,T,N, and B).
//	ds = L/numPts.

// Initiallize.
	var Pti = vm.createPoint( 0.0, 0.0, 0.0 );
	var Ti = vm.createPoint( 1.0, 0.0, 0.0 );
	var Ni = vm.createPoint( 0.0, 1.0, 0.0 );
	var Bi = vm.createPoint( 0.0, 0.0, 1.0 );

//	Put the initial point in the logfactory.	
	logfactory.createInput( 'point' );
	logfactory.setInput( logfactory.numInputs - 1, Pti );

	var si = 0.0;
	var ds = L/numPts;
	var simp = si + ds/2;

	for ( var i = 0; i < numPts; ++i )
	{
		si = i * ds;
		simp = si + ds/2;
		
//		3D Logarithmic Spiral.		
		var kappa = 1 / ( r_0 + (d_r * si ));
		var kappa2 = 1 / ( r_0 + (d_r * simp ));
		var tau = 1/ ( S_0 + (D_S * si ));
		var tau2 = 1/ ( S_0 + (D_S * simp ));		
		
//		Rudy's "rocker".
//		var kappa = 1.0;
//		var kappa2 = kappa;
//		var tau = 1.0 * Math.sin(si);
//		var tau2 = tau;

//		Rudy's "telephone cord".		
//		var kappa = 10.0 * Math.sin(s);
//		var tau = 3.0;

//	alert("to here1")

// Midpoint Values.
//		Tmp = Ti + Kappa * Ni * (ds/2);
			var Nik = scaleVector( Ni, kappa * ds/2 );
		var Tmp = addVectors( Ti, Nik );
			var Bit = scaleVector( Bi, (tau * ds/2) );
			var Tik = scaleVector( Ti, ((-1) * kappa * ds/2) );
		var Nmp = addVectors( Ni, Bit );
		Nmp = addVectors( Nmp, Tik ); 

			var Nit = scaleVector( Ni, ( (-1) * tau * ds/2 ) );
		var Bmp = addVectors( Bi, Nit );
		Tmp = normalize(Tmp);
		Nmp = normalize(Nmp);
		Bmp = normalize(Bmp);
		
//	alert("to here5")
		
// Values at end of interval, (i+1);
//		newT = Ti + (Kappa2*ds) * Nmp;
			var Nmpk = scaleVector( Nmp, kappa2 * ds );
		var newT = addVectors( Ti, Nmpk);
			
			var Bmpt = scaleVector( Bmp, tau2 * ds );
			var Tmpk = scaleVector( Tmp, ((-1) * kappa2 * ds ));
		var newN = addVectors( Ni, Bmpt );
		newN = addVectors( newN, Tmpk ); 
		
			var Nmpt = scaleVector( Nmp, ( (-1) * tau2 * ds )); 
		var newB = addVectors( Bi, Nmpt );
		normalize(newT);
		normalize(newN);
		normalize(newB); 

			var Tis = scaleVector( Ti, ds/6 );
			var Tmps = scaleVector( Tmp, 4*ds/6 );
			var newTs = scaleVector( newT, ds/6 );
		var newPti = addVectors( Pti, Tis );
		newPti = addVectors(newPti, Tmps );
		newPti = addVectors(newPti, newTs );
	
//	alert("to here6")
	
		Ti = newT;
		Ni = newN;
		Bi = newB;
		Pti = newPti;

		logfactory.createInput( 'point' );
		logfactory.setInput( logfactory.numInputs - 1, Pti );
	}
	logfactory.update();	

	return logfactory;
}

function Natural3DSpiral()
{
	var logfactory = Update( moi.ui.commandUI.r_0.numericValue,
	moi.ui.commandUI.d_r.numericValue,
	moi.ui.commandUI.S_0.numericValue,
	moi.ui.commandUI.D_S.numericValue,
	moi.ui.commandUI.L.numericValue,
	moi.ui.commandUI.numPts.integerValue );
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			logfactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'r_0' || dlg.event == 'd_r' || dlg.event == 'S_0' || dlg.event == 'D_S' || dlg.event == 'L' || dlg.event == 'numPts' )
		{
		logfactory.cancel();
		logfactory = Update( moi.ui.commandUI.r_0.numericValue,
		moi.ui.commandUI.d_r.numericValue, moi.ui.commandUI.S_0.numericValue,
		moi.ui.commandUI.D_S.numericValue, moi.ui.commandUI.L.numericValue,
		moi.ui.commandUI.numPts.integerValue );
		}
	}
	
	logfactory.commit();
}

Natural3DSpiral();