//	Inversion3ptsFast Script based upon the Poincaré disk model of Hyperbolic Geometry.
//	Brian McMillin March 17, 2014.

//	This version permits the circle of inversion to be chosen from a 3 point circle selection.
//	The start point of the arc of a circle is picked, with the checkbox option of inverting points.
//	Then endPt and onPt of the arc of a circle are picked, to create the new circle of inversion.
//	Then the center point of the new circle is picked.
//	Then either curves or points are selected to be inverted.
 
//	The circle of inversion may be different from the Poincaré disk circle.
//	An example of this is Hyperbolic tessellation, such as in Escher Circle Limit art.

//	If the user wished to preserve the vertexes of curves/polygons, they should be separated first.
//	Otherwise more irregular curves may be produced, if a point does not happen to be arrayed to some vertex.

//	Checkbox to separate curves...  to do...
//	Checkbox to rejoin curves...  to do...
//	Preservation of color...  to do...
//	If the central curve(s)/polygon(s) is/are set up with closed curves and color, the planar command can be used later, with color.
	
//	The script is primarily intended for the Top View, but some 3d effects may be attained in some cases.

//	Reference: http://www.ma.utexas.edu/users/gilbert/M333L/chp5vers4.pdf 
//	If the Poincaré circle is being used for the inversion, the center would invert to infinity,
//	so curves passing through the center are best avoided.

//	Much code from LineWeb, RingCircle, and BisectorLine, _SpherePoints, etcetera were used.

#include "GetObjects.js"
#include "GetPoint.js"

function DoInversion()
{
	var dialog = moi.ui.commandDialog;
	var ui = moi.ui.commandUI;
	var vM = moi.vectorMath;
	var circlefactory = moi.command.createFactory( 'circle3pt' );	

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var startPt = pointpicker.pt;
	circlefactory.setInput( 0, startPt );
	var radiusPt = startPt;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'FirstPrompt' );
	moi.ui.showUI( 'SecondPrompt' );
	moi.ui.endUIUpdate();	

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var endPt = pointpicker.pt;
	circlefactory.setInput( 1, endPt );	

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SecondPrompt' );
	moi.ui.showUI( 'ThirdPrompt' );
	moi.ui.endUIUpdate();	

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var onPt = pointpicker.pt;
	
	circlefactory.setInput( 2, onPt );
	var invCircleObj = circlefactory.calculate();
//	invCircleObj.selected = true;	
	moi.geometryDatabase.addObjects( invCircleObj );	
	invCircleObj.selected = true;	
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'ThirdPrompt' );
	moi.ui.showUI( 'CenterPrompt' );
	moi.ui.endUIUpdate();	

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var centerPt = pointpicker.pt;
	
	var radiusValue = moi.vectorMath.distance( centerPt, radiusPt );//TEST
//	
	var invPtsFlag = ui.invertPoints.value;	
	if (invPtsFlag)
	{
		moi.ui.beginUIUpdate();
		
		moi.ui.hideUI( 'CenterPrompt' );
		moi.ui.showUI( 'PointsPrompt' );
		moi.ui.endUIUpdate();	

		var ObjectPicker = moi.ui.createObjectPicker();
		ObjectPicker.allowPointObjects();
		if ( !GetObjects( ObjectPicker ) )
		return;

		var objectsToInvert = ObjectPicker.objects;
		objectsToInvert.sortBySelectionOrder();
		//  objectsToInvert = points, in this case.
		
		moi.ui.beginUIUpdate();		
		moi.ui.hideUI( 'PointsPrompt' );
		moi.ui.endUIUpdate();
	}
	else
	{
		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'CenterPrompt' );
		moi.ui.showUI( 'CurvesPrompt' );
		moi.ui.endUIUpdate();	

		var ObjectPicker = moi.ui.createObjectPicker();
		ObjectPicker.allowStandaloneCurves();
		if ( !GetObjects( ObjectPicker ) )
		return;
		
		var objectsToInvert = ObjectPicker.objects;
		objectsToInvert.sortBySelectionOrder();
		//  objectsToInvert = curves, in this case.		

		moi.ui.beginUIUpdate();
		moi.ui.hideUI( 'CurvesPrompt' );
		moi.ui.showUI( 'OptionsPrompt' );
		moi.ui.showUI( 'Options' );
		moi.ui.endUIUpdate();	
	}

	var objectsList = moi.ui.commandUI.Update( objectsToInvert, invPtsFlag, centerPt, radiusValue, ui.numpoints.integerValue );
	moi.geometryDatabase.addObjects(objectsList);

	while ( true )
	{
		dialog.waitForEvent();			
		if ( dialog.event == 'done' ) 
		{
			moi.geometryDatabase.removeObjects( invCircleObj );		
			break;
		}	
			
		if ( dialog.event == 'pRadius' || dialog.event == 'numpoints' )
		{
			var radiusValue = ui.pRadius.numericValue;
			moi.geometryDatabase.removeObjects(objectsList);			
			var objectsList = moi.ui.commandUI.Update( objectsToInvert, invPtsFlag, centerPt, radiusValue, ui.numpoints.integerValue );
			moi.geometryDatabase.addObjects(objectsList);
		}
		else
		{
			moi.geometryDatabase.removeObjects(objectsList);
			moi.geometryDatabase.removeObjects( invCircleObj );			
			return
		}
	}	
}

DoInversion();
