//	InvertPoints Script by Brian McMillin March 9, 2014.
//	This script does Inversion for selected points.
//  It is a modification of the Inversion script.
//	Code from scripts SavePointFile, LineWeb, RingCircle, and BisectorLine was used.	

#include "GetPoint.js"
#include "GetObjects.js"

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function dotproduct( v0, v1 )
{
	return (v0.x * v1.x) + (v0.y * v1.y) + (v0.z * v1.z);
}

function InvertPt( centerPt, evalPt, pR )
{
	var vM = moi.vectorMath;
	var dirOP = vM.makeVector(centerPt, evalPt);
	var lensqOP = lengthsq( dirOP );
	var pRsq = pR * pR;
	var mult = pRsq / lensqOP;
	return vM.createPoint( centerPt.x + dirOP.x * mult, centerPt.y + dirOP.y *mult, centerPt.z + dirOP.z * mult );	
}

function lengthsq( v )
{
	return dotproduct( v, v );
}

function Update( Points, centerPt, pRadius, factories )
{
	Cancel( factories );

	var vM = moi.vectorMath;
	var circleFactory = moi.command.createFactory( 'circle' );
	var FrameOrigin = centerPt;
	var FrameXAxis = vM.createPoint( 1, 0, 0 );
	var FrameYAxis = vM.createPoint(0, 1, 0 );
	var Frame = vM.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	circleFactory.setInput( 1, Frame );
	circleFactory.setInput( 0, true );
	circleFactory.setInput( 3, pRadius );
	circleFactory.update();

	//	Invert each point not equal to centerPt, to a point.
	//  Points is a MoI object list(?), not just an array of points as (x,y,z)(?)
	//	So x, y, and z must be extracted.(?)	
	for ( var i=0; i < Points.length; ++i )
	{
		x = Points.item(i).pt.x;
		y = Points.item(i).pt.y;
		z = Points.item(i).pt.z;
		var evalPt = vM.createPoint( x, y, z );	
		//	The point must not be the same as the origin.
		if ( x != 0 || y != 0 || z != 0 )
		{
			var factory = moi.command.createFactory( 'point' );
			var pt = InvertPt( centerPt, evalPt, pRadius );
			factory.setInput( 0, pt );
			factories.push( factory );
		}
	}
	for ( var i = 0; i < factories.length; ++i )
	{
		factories[i].update();
	}	
}

function DoInversion()
{
	var ui = moi.ui.commandUI;
	var vM = moi.vectorMath;

	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
	return;
	var centerPt = pointpicker.pt;	

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'PickPrompt' );
	moi.ui.showUI( 'SelectPrompt' );
	moi.ui.endUIUpdate();	

	var ObjectPicker = moi.ui.createObjectPicker();
	ObjectPicker.allowPointObjects();
	if ( !GetObjects( ObjectPicker ) )
		return;

	var Points = ObjectPicker.objects;
	Points.sortBySelectionOrder();

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
		
	moi.ui.endUIUpdate();	

	var factories = new Array();
	
	Update( Points, centerPt, ui.pRadius.numericValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'pRadius' )
			Update( Points, centerPt, ui.pRadius.numericValue, factories );
	}
			Commit( factories );
}

DoInversion();
