// Script to graph 3 of the Jacobian Elliptic functions, cn, dn, and sn, in Moi.
// By Brian McMillin, November, 2014.
// amplitude, am is also graphed.
// cn resemples cosine. sn resembles sine. dn is doubly periodic.

var g_style_index = 1;// second color may be RED

function AssignColors2( factory )
{
	var styles = moi.geometryDatabase.getObjectStyles();

	var curves = factory.getCreatedObjects();
	for ( var i = 0; i < curves.length; ++i )
	{
		if ( g_style_index == styles.length )
		{
			g_style_index = 0;
		}
		var curve = curves.item(i);
		curve.styleIndex = g_style_index;
	}
++g_style_index;	
}

function AssignColors( factory )//Not Used
{
	var styles = moi.geometryDatabase.getObjectStyles();
	var curves = factory.getCreatedObjects();
	for ( var i = 0; i < curves.length; ++i, ++g_style_index )
	{
		if ( g_style_index == styles.length )
		{
			g_style_index = 0;
		}
		var curve = curves.item(i);
		curve.styleIndex = g_style_index;
	}
}

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function cosh(x)
{
	return (Math.exp(x) + Math.exp(-x))/2.0;
}

function sinh(x)
{
	return (Math.exp(x) - Math.exp(-x))/2.0;
}

function tanh(x)
{
	return (Math.exp(x) - Math.exp(-x))/(Math.exp(x) + Math.exp(-x));
}

// Returns the incomplete elliptic integral of the first kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei1F(angle, kk)
{
	var u = angle;
	var cosu2 = Math.cos(u);
	cosu2 *= cosu2;
	var sinu2 = Math.sin(u);
	sinu2 *= sinu2;	
	var result = Math.sin(u) * RF(cosu2, (1 - kk * sinu2), 1);
    return result;
}

function Jacobi_am(uValue,m)
{
	var asav = new Array();
	var csav = new Array();	
	var k = Math.sqrt(m);
	var m1 = 1.0 - m;	
	var phi;
	var PI_2 = Math.PI/2;	
	var tol = 10.0e-11;
 	
    if (uValue == 0.0) return 0.0;

	if (m < tol) //A&S 16.13.4
	{
		var result = uValue - 0.25*m*(uValue - 0.5 * Math.sin(2.0*uValue));
		return result;
	}	

    if (m1 < tol) //A&S 16.15.4
	{
        var t = tanh(uValue);
        result = Math.asin(t) + 0.25 * m1 * (t - uValue * (1. - t*t)) * cosh(uValue);	
		return result;		
	}
	
	// Check special case k = 1
	if ( k == 1.0 ) return 2.0 * Math.atan( Math.exp(uValue) ) - PI_2;

	// If k > 1, then perform a Jacobi modulus transformation. NOT DONE

	asav[0] = 1.0;
	b = Math.sqrt(m1);
	csav[0] = k; 
	var n2 = 1.0;
	var i = 0;

	while( Math.abs( (csav[i]/asav[i]) ) > tol )
	{
		if( i > 10 )
		{
		/*mtherr( "ellpjf", OVERFLOW );*/
		break;
		}
		var a = asav[i];
		++i;
		csav[i] = 0.5 * ( a - b );
		var t = Math.sqrt( a * b );
		asav[i] = 0.5 * ( a + b );
		b = t;
		n2 += n2;
	}
	/* backward recurrence */
	// Find phi_subzero, starting at a[n] and c[n], and decrementing to n=0.
	// See paragraph 22.20.2 and .3 and .4 of nist website.
	var phi = n2 * asav[i] * uValue;
	do
	{
		t = csav[i] * Math.sin(phi) / asav[i];
		b = phi;
		phi = 0.5 * (Math.asin(t) + phi);
	}
	while( --i );
	return phi;
}

//  Computes the R_F from Carlson symmetric form 
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
function RF( X, Y, Z )
{
	//var X, Y, Z;
	var result, A, lamda, dx, dy, dz;
	var MinError = 0.0000001;

	do
	{
       lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);

       X = (X + lamda) / 4;
       Y = (Y + lamda) / 4;
       Z = (Z + lamda) / 4;

       A = (X + Y + Z) / 3;

       dx = (1 - X / A);
       dy = (1 - Y / A);
       dz = (1 - Z / A);
	}
    while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

    var E2, E3;
    E2 = dx * dy + dy * dz + dz * dx;
    E3 = dy * dx * dz;

    //http://dlmf.nist.gov/19.36#E1
    result = 1 - (1 / 10) * E2 + (1 / 14) * E3 + (1 / 24) * E2 * E2 - (3 / 44) * E2 * E3 - (5 / 208) * E2 * E2 * E2  + (3 / 104) * E3 * E3 + (1 / 16) * E2 * E2 * E3;

    result *= (1 / Math.sqrt(A));
    return result;
}

function FrameCreate( BasePoint )
{
	//	Create a frame in Top View with center at BasePoint
	var vM = moi.vectorMath;	
	var FrameOrigin = BasePoint;
	var FrameXAxis = vM.createPoint(1, 0, 0 );
	var FrameYAxis = vM.createPoint( 0, 1, 0 );
	var frame = vM.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	return frame;
}

// Label the curves am, cn, dn, and sn, by color
function TextLabel( frame, text )
{
	var ui = moi.ui;
	var factory = moi.command.createFactory( 'text' );
	ui.commandUI.PopulateFonts( factory );	

	var font = 'Arial';
	var type = 'surfaces';
	var height = '0.2';

	factory.disableUpdate( true );
	factory.setInput( 0, frame );
	factory.setInput( 1, text );	
	factory.setInput( 2, font );	
//	factory.setInput( 3, bold );	
//	factory.setInput( 4, italic );
	factory.setInput( 5, type );
	factory.setInput( 6, height );
//	factory.setInput( 7, extrusion );	
	factory.setInput( 8, false ); // Disable preview mode	
	
	factory.disableUpdate( false );
	factory.update();
	AssignColors2( factory )
	return factory
}

function Update( kmod, numpoints, amfactories, am2factories, cnfactories, dnfactories, snfactories, amlabel, cnlabel, dnlabel, snlabel )
{
	Cancel( amfactories );
	Cancel( am2factories );	
	Cancel( cnfactories );
	Cancel( dnfactories );
	Cancel( snfactories );
	Cancel( amlabel );
	Cancel( cnlabel );
	Cancel( dnlabel );	
	Cancel( snlabel );	
	
	var vM = moi.vectorMath;//abbreviations
	var gd = moi.geometryDatabase;
	var UI = moi.ui.commandUI;
	
	var amfactory = moi.command.createFactory( 'interpcurve' );
	var am2factory = moi.command.createFactory( 'interpcurve' );	
	var cnfactory = moi.command.createFactory( 'interpcurve' );
	var dnfactory = moi.command.createFactory( 'interpcurve' );
	var snfactory = moi.command.createFactory( 'interpcurve' );

	var uValue, amValue, cnValue, dnValue, snValue;
	var iu, ptam, ptcn, ptdn, ptsn;
		
	var z = 0.0;
	var m = kmod * kmod; // elliptic parameter.
	var PIo2 = Math.PI / 2;
	var Kofkmod =  ei1F( PIo2, m ); // Complete elliptic integral. = K( PI/2 | m) = K( PI/2, kmod)
	// cn and sn are periodic over 4K.  dn is periodic over 2K.
	var uMax = 4 * Kofkmod; // Max abscissa value.
	// Make numpoints odd.
	if (numpoints%2 == 0)
		numpoints += 1;
	
	for ( var i = 0; i < numpoints; ++i )
	{
		iu = i / (numpoints-1);
		uValue = uMax * iu;
		amValue = Jacobi_am(uValue,m);// phi
		cnValue = Math.cos(amValue);
		dnValue = Math.sqrt( 1 - m * Math.pow(Math.sin(amValue),2 ));
		snValue = Math.sin(amValue);

		// Split am curve at 2Kofkmod.  |am(u,k)| <= PI.
		if (amValue <= Math.PI)
		{
			amValue2 = amValue - Math.PI;
			var uValue2 = uValue + (2 * Kofkmod);
			ptam = moi.vectorMath.createPoint( uValue, amValue, z );
			ptam2 = moi.vectorMath.createPoint( uValue2, amValue2, z );

			amfactory.createInput( 'point' );
			amfactory.setInput( amfactory.numInputs - 1, ptam );
		
			am2factory.createInput( 'point' );
			am2factory.setInput( am2factory.numInputs - 1, ptam2 );			
		}

		ptcn = moi.vectorMath.createPoint( uValue, cnValue, z );
		ptdn = moi.vectorMath.createPoint( uValue, dnValue, z );
		ptsn = moi.vectorMath.createPoint( uValue, snValue, z );

		cnfactory.createInput( 'point' );
		cnfactory.setInput( cnfactory.numInputs - 1, ptcn );

		dnfactory.createInput( 'point' );
		dnfactory.setInput( dnfactory.numInputs - 1, ptdn );

		snfactory.createInput( 'point' );
		snfactory.setInput( snfactory.numInputs - 1, ptsn );
	}
	amfactories.push( amfactory );
	am2factories.push( am2factory );	
	cnfactories.push( cnfactory );
	dnfactories.push( dnfactory );
	snfactories.push( snfactory );

	for ( var i = 0; i < amfactories.length; ++i )
		{
			amfactories[i].update();
			AssignColors2( amfactories[i] );
			--g_style_index;// go back to (keep) same color.
		}
	var text = 'am';
	var BasePoint = vM.createPoint(0, -0.5, 0 );
	var frame = FrameCreate( BasePoint )
	var textfactory = TextLabel( frame, text );
	amlabel.push( textfactory );
	--g_style_index;	

	for ( var i = 0; i < am2factories.length; ++i )
		{
			am2factories[i].update();
			AssignColors2( am2factories[i] );
		}		
		
	for ( var i = 0; i < cnfactories.length; ++i )
		{
			cnfactories[i].update();
			AssignColors2( cnfactories[i] );
			--g_style_index;			
		}
	var text = 'cn';
	var BasePoint = vM.createPoint(0, -1.0, 0 );
	var frame = FrameCreate( BasePoint )
	var textfactory = TextLabel( frame, text );
	cnlabel.push( textfactory );
		
	for ( var i = 0; i < dnfactories.length; ++i )
		{
			dnfactories[i].update();
			AssignColors2( dnfactories[i] );
			--g_style_index;			
		}		
	var text = 'dn';
	var BasePoint = vM.createPoint(0, -1.5, 0 );
	var frame = FrameCreate( BasePoint )
	var textfactory = TextLabel( frame, text );
	dnlabel.push( textfactory );
	
		
	for ( var i = 0; i < snfactories.length; ++i )
		{
			snfactories[i].update();
			AssignColors2( snfactories[i] );
			--g_style_index;			
		}
	var text = 'sn';
	var BasePoint = vM.createPoint(0, -2.0, 0 );
	var frame = FrameCreate( BasePoint )
	var textfactory = TextLabel( frame, text );
	snlabel.push( textfactory );
}

function JacobianGraph()
{
	var UI = moi.ui.commandUI; //Abbreviations
	var ui = moi.ui;

	var factory = moi.command.createFactory( 'text' );
	ui.commandUI.PopulateFonts( factory ); // function in htm file
	
	var amfactories = new Array();
	var am2factories = new Array();	
	var cnfactories = new Array();
	var dnfactories = new Array();
	var snfactories = new Array();
	var amlabel = new Array();	
	var cnlabel = new Array();
	var dnlabel = new Array();	
	var snlabel = new Array();	
	
	Update( UI.kmod.numericValue, UI.numpoints.integerValue, amfactories, am2factories, cnfactories, dnfactories, snfactories, amlabel, cnlabel, dnlabel, snlabel )

	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( amfactories );
			Cancel( am2factories );			
			Cancel( cnfactories );			
			Cancel( dnfactories );			
			Cancel( snfactories );
			Cancel( amlabel );			
			Cancel( cnlabel );
			Cancel( dnlabel );
			Cancel( snlabel );			
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'kmod' || dlg.event == 'numpoints' )
		{
		g_style_index = 1;//reset color start index	
		Update( UI.kmod.numericValue, UI.numpoints.integerValue, amfactories, am2factories, cnfactories, dnfactories, snfactories, amlabel, cnlabel, dnlabel, snlabel )
		}
	}
	Commit( amfactories );
	Commit( am2factories );	
	Commit( cnfactories );
	Commit( dnfactories );	
	Commit( snfactories );
	Commit( amlabel );	
	Commit( cnlabel );
	Commit( dnlabel );	
	Commit( snlabel );	
}

JacobianGraph();