// config: norepeat noautolaunch

#include "GetObjects.js"
#include "WaitForDialogDone.js"

function DoMake2D4Views()
{
	var ui = moi.ui;

	var objectpicker = ui.createObjectPicker();
	objectpicker.allowBReps();
	objectpicker.allowCurves();
	if ( !GetObjects( objectpicker ) )
		return;

	ui.beginUIUpdate();
	ui.hideUI( 'SelectObjectsPrompt' );
	ui.showUI( 'OptionsPrompt' );
	ui.showUI( 'Make2DOptions' );
	ui.endUIUpdate();
	
	if ( !WaitForDialogDone() )
		return;

	ui.beginUIUpdate();
	ui.hideUI( 'OptionsPrompt' );
	ui.hideUI( 'Make2DOptions' );
	ui.hideUI( 'done' );
	ui.hideUI( 'cancel' );
	ui.endUIUpdate();
		
	var output = [];
	var views = [ 'Right', 'Front', '3D', 'Top' ];
	var is_perspective = moi.ui.mainWindow.viewpanel.getViewport('3D').projection == 'Perspective';

	
	// Generate 2D curves.
	
	for ( var i = 0; i < 4; ++i )
	{
		var factory = moi.command.createFactory( 'make2d' );
		factory.setInput( 0, objectpicker.objects );
		factory.setInput( 1, views[i] );
		factory.setInput( 2, ui.commandUI.resolution.value );
		factory.setInput( 3, ui.commandUI.hiddenlines.value );
		
		output[i] = factory.calculate();
		factory.cancel();
	}
	
	
	// Gather the bounding boxes of each 2D curve set.
	
	var bounds = [];
	var w = 0, h = 0;
	
	for ( var i = 0; i < 4; ++i )
	{
		bounds[i] = output[i].getHighAccuracyBoundingBox();
		
		if ( views[i] == '3D' && is_perspective )
			continue;		
		
		w = Math.max( w, bounds[i].xLength );
		h = Math.max( h, bounds[i].yLength );
	}
	
	// A perspective 3D view does not generate scale-preserving output, scale its output to fit within the other view sizes.
	
	if ( is_perspective )
	{
		for ( var i = 0; i < 4; ++i )
		{
			if ( views[i] != '3D' )
				continue;
				
			var tarw = bounds[i].xLength;
			var tarh = bounds[i].yLength;
				
			if ( w == 0 || h == 0 || tarw == 0 || tarh == 0 )
				continue;
				
			var target_aspect = w / h;
			var source_aspect = tarw / tarh;
			var scale = 1.0;
			
			if ( target_aspect < source_aspect )
				scale = w / tarw;			
			else
				scale = h / tarh;
			
			var factory = moi.command.createFactory( 'scale' );
			factory.setInput( 0, output[i] );
			factory.setInput( 1, bounds[i].center );
			factory.setInput( 2, scale );
			
			output[i] = factory.calculate();
			factory.cancel();
		}
	}

	
	// Calculate the center points of each target location.	
	
	var cenx = [];
	var ceny = [];
	
	cenx[0] = w * 0.5;
	ceny[0] = h * 1.7;
	
	cenx[1] = w * 1.7;
	ceny[1] = h * 1.7;
	
	cenx[2] = w * 0.5;
	ceny[2] = h * 0.5;
	
	cenx[3] = w * 1.7;
	ceny[3] = h * 0.5;
	
	
	// Move each set of curves from their current center point to their target center point.		
	
	objectpicker.objects.unlockSelection();
	moi.geometryDatabase.deselectAll();
	
	for ( var i = 0; i < 4; ++i )
	{
		var factory = moi.command.createFactory( 'move' );
		factory.setInput( 0, output[i] );
		factory.setInput( 1, bounds[i].center );
		factory.setInput( 2, moi.vectorMath.createPoint( cenx[i], ceny[i], 0 ) );
		
		factory.update();
		factory.getCreatedObjects().setProperty( 'selected', true );
		factory.commit();
	}
}

DoMake2D4Views();
