#include "GetObject.js"

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function CreatePointObject( pt )
{
	var factory = moi.command.createFactory( 'point' );
	factory.setInput( 0, pt );
	
	var result = factory.calculate();
	var ptobj = result.item(0);
	
	return ptobj;
}

function WrapWithObjectList( obj )
{
	var list = moi.geometryDatabase.createObjectList();
	list.addObject( obj );
	return list;
}

function MakePointsAlongCurve( crv, numpoints )
{
	var factory = moi.command.createFactory( 'arraycurve' );
	
	var startpt = crv.getStartPt();

	factory.setInput( 0, WrapWithObjectList(CreatePointObject(startpt)) );
	factory.setInput( 1, crv );
	factory.setInput( 2, 'NumItems' );
	factory.setInput( 3, numpoints );
	factory.setInput( 4, 0.0 );
	factory.setInput( 5, 'Freeform' );
	
	var result = factory.calculate();
	
	var points = new Array();
	points.push( startpt );	
	
	for ( var i = 0; i < result.length; ++i )
		points.push( result.item(i).pt );
		
	return points;
}

function Update( edge, face, factories )
{
	Cancel( factories );
	
	var points = MakePointsAlongCurve( edge, moi.ui.commandUI.isocount.value );
	
	for ( var i = 0; i < points.length; ++i )
	{
		factory = moi.command.createFactory( 'isocurve' );
		factory.setInput( 0, face );
		factory.setInput( 1, points[i] );
		factory.setInput( 2, moi.ui.commandUI.dir.value );
		factory.update();
		
		factories.push( factory );
	}
}

function DoMultiIso()
{
	var objectpicker = moi.ui.createObjectPicker();
	
	objectpicker.allowEdgeCurves();
	if ( !GetObject( objectpicker ) )
		return;
		
	var edge = objectpicker.objects.item(0);
	
	var faces = edge.getFacesOfEdge();
	if ( faces.length != 1 )
		return;
		
	var face = faces.item(0);
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'options' );
	moi.ui.endUIUpdate();
	
	var factories = new Array();
	Update( edge, face, factories );
	
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
			return false;
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'dir' || dlg.event == 'isocount' )
			Update( edge, face, factories );
	}
	
	Commit( factories );
}

DoMultiIso();
