//	PolygonFaceArea is a script which computes the area of a polygon face.
//	By Brian McMillin, December 2012
//	

#include "GetObjects.js"
#include "WaitForDialogDone.js"

function Alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function CrossProduct( v0, v1 )
{
	var x = (v0.y * v1.z) - (v0.z * v1.y);
	var y = (v0.z * v1.x) - (v0.x * v1.z);
	var z = (v0.x * v1.y) - (v0.y * v1.x);
	return moi.vectorMath.createPoint( x, y, z );
}

function LengthSq( vec )
{
	return (vec.x * vec.x) + (vec.y * vec.y) + (vec.z * vec.z);
}

function Length( vec )
{
	return Math.sqrt( LengthSq(vec) );
}

function DoPolygonFaceArea()
{
	var gd = moi.geometryDatabase;
	var vm = moi.vectorMath;
	var face = new Array();
	var edges = new Array();
	var newedge = new Array();
	var vertexPoints = new Array();
	var total = new Array();
	var originpt = vm.createPoint(0,0,0);

	var objectpicker = moi.ui.createObjectPicker();

	objectpicker.allowFaces();	
	objectpicker.min = objectpicker.max = 1;

	if ( !GetObjects( objectpicker ) )
		return;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'FirstSelectPrompt' );
	moi.ui.endUIUpdate();

	var faces = objectpicker.objects;
	if ( faces.length != 1 )
		return;
		
//  Used MG script "switch selection from a face to its edges"
//	Used MG script "DuplicateBorders"
//	Save the start points of the perimeter lines.
	face = faces.item(0);
	edges = face.getEdges();	
	for ( var iEdge = 0; iEdge < edges.length; ++iEdge )
	{
		newedge = edges.item(iEdge).clone();
		moi.geometryDatabase.addObject( newedge );
		newedge.selected = true;
		vertexPoints.push(newedge.getStartPt());
	}
	vertexPoints.push(newedge.getEndPt());

//	For an n sided polygon, n start points are use, plus the last point = first point.
//	The Area * 2 = the sum of the cross product of the vertex point vectors.
	var AreaX2 = 0.0;
	var ax = 0.0;
	var ay = 0.0;
	var az = 0.0;
	
 	for ( var i = 0; i < (vertexPoints.length-1); ++i )
		{
		var point_end0 = vertexPoints[i]; 
		var point_end1 = vertexPoints[i+1];
		var dir0 = vm.makeVector( originpt, point_end0 );
		var dir1 = vm.makeVector( originpt, point_end1 );
		var areaPoint = CrossProduct( dir0, dir1 );		
		
		ax = ax + areaPoint.x;
		ay = ay + areaPoint.y;
		az += areaPoint.z;
		}
	
	var AreaSumPt = moi.vectorMath.createPoint( ax,ay,az );	
	var areaVector = vm.makeVector( originpt, AreaSumPt );	
	AreaX2 = Length(areaVector);

//	Used MG Angle script.	
	moi.ui.commandUI.Area.value = AreaX2 / 2;
	moi.ui.beginUIUpdate();
	moi.ui.showUI( 'AreaTable' );
	moi.ui.showUI( 'done' );
	moi.ui.endUIUpdate();
	
	WaitForDialogDone();
}

DoPolygonFaceArea();
	

		


