//	PropDesignMoiScript, a program to design propellers.
//	The impetus to create this script began with Anthony Falzone.
//	This script is based upon the PROP_DESIGN_GEO fortran program of Anthony Falzone,
//	which is in the Public Domain.  http://propdesign.weebly.com/
//	See MoI discussion forum topic "Anyone wish to develop a custom script?"  
//	DRAFT by Brian McMillin December 19 - 31, 2011, - January 8, 2012.
//  (Modified from Michaels ToroidalHelix and LineWeb scripts, MoI program)
//	Help with fortran conversion to javascript: http://www.see.ed.ac.uk/~jwp/JavaScript/programming/notes.html
//	Note: for the most part, Capitalization of Fortran variables was kept.
//	The AirFoil will have a small radius at the te (trailing edge), 
//	and in between will be "through point" type MoI interpcurve(s).
//	The leading edge is now a conic.
//	The Rails are also "through point" MoI interpcurve type curves.
//	Invaluable help from Michael on Object lists, intersection, AND...

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( ArrayArgument )
{
	for ( var i = 0; i < ArrayArgument.length; ++i )
		ArrayArgument[i].cancel();
	ArrayArgument.length = 0;
}

function Commit( ArrayArgument )
{
	for ( var i = 0; i < ArrayArgument.length; ++i )
		ArrayArgument[i].commit();
}

//	Copied from SpurGearProfile.js, to make join list of curves.
function concat( list )
{
	for ( var i = 1; i < arguments.length; i++ )
	{
		var arglist = arguments[i];
	
		for ( var j = 0; j < arglist.length; j++ )
			list.addObject( arglist.item(j) );
	}

	return list;
}

//	Copied from SpurGearProfile.js, Note, used this function only once.  It COULD be
//	used more, to create several different moi Factory(s), and shrink the code.
//	It was not used more, in order to demonstrate the various factory "methods." 

function factory( factoryname )
{
	var f = moi.command.createFactory( factoryname );

	for ( var i = 1; i < arguments.length; i++ )
		f.setInput( i - 1, arguments[i] );

	var objects = f.calculate();
	f.cancel();

	return objects;
}

function Update( AFXTAB, AFYTAB, BETA, CHORD, DELTA, DLP, DPHI, DR, PHI, R, X, Y, DX, DY, TIP, PITCH, RADIUS, ROCR, PHIMAX, CDStype, RDStype, SDStype, LeadingEdgefactory, QtrChordfactory, TrailingEdgefactory, ControlPointsfactories, obj )
{
	Cancel( LeadingEdgefactory );
	Cancel( QtrChordfactory );
	Cancel( TrailingEdgefactory );
	Cancel( ControlPointsfactories );
	
//	No need for MAXSIZ in javascript, arrays are automatically adjusted in size.	
//	var MAXSIZ = 1000;

//	NOT used: var PI = 3.14159265358979320.  Did use math.PI;

//	Calculating the CHORD DISTRIBUTION array.
//	Array subscripts actually start at zero, but the first element can be ignored...(see weblink at top.)
//	...carefully. (keep first element out of factories...)
	for (var J = 1; J <= TIP; ++J )
	{
//	Chord distribution switch (CDS), is 'Constant', or else 'Elliptical'.
//	Note, in javascript, double equal sign, == is for true/false.
	if ( CDStype == 'Constant' ) 
		{
			CHORD[J] = RADIUS / ROCR;
		}
		else if ( J == TIP )
		{
			CHORD[J] = 0.0;
		}
		else
		{
		CHORD[J] = Math.sqrt( Math.pow((RADIUS / ROCR ), 2) * (1.0 - ( Math.pow(((( J - 1.0 ) / (TIP - 1.0 )) * RADIUS ), 2 ) / Math.pow( RADIUS, 2))));
		}
	}

//	PHIMAX, Determines the Amount of Blade Sweep
	PHIMAX = PHIMAX * (Math.PI / 180 );

//	(Using notation similar to LineWeb script.)
//	Rotation Direction Switch (RDS): (Fortran 'Clockwise' was 1, 'CounterCW' was 2)
//	RDStype is 'Clockwise', or else 'CounterCW'

//	(Using notation similar to LineWeb script.)
//	Sweep Direction Switch (SDS): (Fortran 'BackSweep' was 1, 'ForwardSweep' was 2)
//	SDStype is 'BackSweep', or else 'ForwardSweep'

//	Propeller Geometry at Boundaries:
//	Ignore first element (element zero) in arrays, set second element (element 1) in arrays.
	R[1] = 0.0;
	PHI[1] = 0.0;
	X[1] = 0.0;
	Y[1] = 0.0;
	DX[1] = 0.0;
	DY[1] = 0.0;
	DPHI[1] = 0.0;
	DR[1] = 0.0;
	DLP[1] = 0.0;
	DELTA[1] = 0.0;
	BETA[1] = 90.0 * ( Math.PI / 180.0 );
	R[TIP] = RADIUS;

//	Calculate Sweep Angle, (DELTA), and Geometric Angle of Attack, (BETA).

	for (var J = 2; J <= TIP; ++J )
	{
		R[J] = R[J-1] + ( R[TIP] / ( TIP - 1.0 ) );
		PHI[J] = PHI[J-1] + ( PHIMAX / ( TIP - 1.0 ) );
		X[J] = R[J] * Math.cos( PHI[J] );
		Y[J] = R[J] * Math.sin( PHI[J] );
		DX[J] = X[J] - X[J-1];
		DY[J] = Y[J] - Y[J-1];
		DPHI[J] = PHI[J] - PHI[J-1];
		DR[J] = R[J] * Math.cos( DPHI[J] ) - R[J-1];
		DLP[J] = Math.sqrt(Math.pow(Math.abs(DX[J]), 2) + Math.pow(Math.abs(DY[J]), 2));
		DELTA[J] = Math.acos( DR[J] / DLP[J] );
		
		if ( DELTA[J] >= ( Math.PI / 2.0 ) )
		{
			alert("Reduce PHI MAX it is too high. (DELTA >=90 degrees.")
		}
		ARG1 = PITCH;
		ARG2 = 2.0 * Math.PI * R[J] * Math.cos( DELTA[J] );
		if (ARG2 < 0.0 || ARG2 > 0.0)
		{
			BETA[J] = Math.atan( ARG1 / ARG2);
		}
		else
		{
			BETA[J] = 0.0
		}
	}

//	Airfoil Points:
//	Note, javascript array element index starts at zero.	
	
	var x = 0.0;
	var vM = moi.vectorMath;  //abbreviation
	var gd = moi.geometryDatabase;  //abbreviation

//Second time through, remove previous airFoilProfile.
	if ( obj != null )
		gd.removeObjects( obj );

//	Stuck this in here to mimic SpurGearProfile.  Do not know what it does.		
	if ( TIP == 0 || PITCH == 0 || !isFinite( TIP ) )
		return null;	
		
	var y_tanPt1 = ( CHORD[1] / 100.0 ) * AFYTAB[0] * 1000.0;
	var y_negY_tanPt1 = (-1) * y_tanPt1;
	var z_tanPt1 = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * AFXTAB[0] )) * 1000.0;
	var tanPt1 = vM.createPoint( x, y_tanPt1, z_tanPt1 );
	var negY_tanPt1 = vM.createPoint( x, y_negY_tanPt1, z_tanPt1 );

	var y_tanPt2 = ( CHORD[1] / 100.0 ) * AFYTAB[24] * 1000.0;
	var y_negY_tanPt2 = (-1) * y_tanPt2;
	var z_tanPt2 = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * AFXTAB[24] )) * 1000.0;
	var tanPt2 = vM.createPoint( x, y_tanPt2, z_tanPt2 );
	var negY_tanPt2 = vM.createPoint( x, y_negY_tanPt2, z_tanPt2 );	

	//  Note: z value of tantanPt has already been reverse calculated for AFXTAB value 	
	//	for the default AirFoil profile.  It comes out negative.
	//	AFXTAB_tantan = (250 * (RADIUS/ROCR) - z_tantanPt) / (10 * RADIUS/ROCR)
	var AFXTAB_tantan = -0.57937;
	var y_tantanPt = ( CHORD[1] / 100.0 ) * 0.0 * 1000.0;
	var z_tantanPt = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * AFXTAB_tantan )) * 1000.0;
	
	var tantanPt = vM.createPoint(x,y_tantanPt,z_tantanPt);

	var ypt_leStart = ( CHORD[1] / 100.0 ) * 0.0 * 1000.0;
	var zpt_leStart = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * 0.0 )) * 1000.0;
	var pt_leStart = vM.createPoint( x, ypt_leStart, zpt_leStart );

	var ypt_teArc3pt = ( CHORD[1] / 100.0 ) * 0.0 * 1000.0;
	var zpt_teArc3pt = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * 100.0 )) * 1000.0;
	var pt_teArc3pt = vM.createPoint( x, ypt_teArc3pt, zpt_teArc3pt );
	
	//	Create the curve.
	var curvefactory = moi.command.createFactory( 'interpcurve' );
	for (var M3 = 0; M3 < 25; ++M3 )
	{
	AFX = 0.0;
	AFY = ( CHORD[1] / 100.0 ) * AFYTAB[M3] * 1000.0;
	AFZ = (0.250 * CHORD[1] - ( ( CHORD[1] / 100.0 ) * AFXTAB[M3] )) * 1000.0;
	
		var pt = vM.createPoint( AFX, AFY, AFZ );	
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );
	//	Save the control points.
	var pointfactory = moi.command.createFactory( 'point' );		
	pointfactory.setInput( 0, pt );
	ControlPointsfactories.push( pointfactory );		
	}
	
	//  Create curve ObjectList, for mirror input and join.
	var curve = curvefactory.calculate();
	curvefactory.cancel();
	
	//	To Mirror curve, using the CoordinateFrame:
	var FrameOrigin = vM.createPoint( 0, 0, 0 );
	var FrameXAxis = vM.createPoint(0, 1, 0 );
	var FrameYAxis = vM.createPoint( 0, 0, 1 );
	//	Note, this is the RIGHT view in MoI. (z up axis and y rightfacing axis.)
	var Frame = vM.createFrame( FrameOrigin, FrameXAxis, FrameYAxis );
	var mirrorfactory = moi.command.createFactory( 'mirror' );
	mirrorfactory.setInput( 0, curve );
	mirrorfactory.setInput( 1, Frame );
	//	Set End pt.  (FrameYAxis pt on z axis works fine. (It really is a z point.)
	mirrorfactory.setInput( 2, FrameYAxis );
	//	Prevent the temporary UI line on z axis from showing up.
	mirrorfactory.disableUIGeometry();
	//	Create mirror ObjectList, for join.
	var mirror = mirrorfactory.calculate();
	mirrorfactory.cancel();
	
//	Create the leading edge ellipse conic.
	var conicFactory = moi.command.createFactory( 'conic' );
	// Start and End points.
	conicFactory.setInput(0, tanPt1 );
	conicFactory.setInput(1, negY_tanPt1 );
	// Apex point.
	conicFactory.setInput(2, tantanPt );
	// Through point.
	conicFactory.setInput(3, pt_leStart );
	//	Prevent the temporary UI tangent lines, of conic, from showing up.
	conicFactory.disableUIGeometry();
	//	Create conic ObjectList, for join. 
	var conic = conicFactory.calculate();
	conicFactory.cancel();	
	
//	Trailing edge 3 point arc.
	var arcFactory2 = moi.command.createFactory( 'arc3pt' );	
	arcFactory2.setInput( 0, tanPt2 );
	arcFactory2.setInput( 1, negY_tanPt2 );
	arcFactory2.setInput( 2, pt_teArc3pt );
	var arc = arcFactory2.calculate();
	arcFactory2.cancel();

	//	Combine the 4 Object lists into one, in order to do a join factory.
	var airFoilProfile = factory( 'join', concat( curve, arc, mirror, conic) );
	gd.addObjects( airFoilProfile );
	airFoilProfile.lockSelection();
	// a return airFoilProfile is located at end of Update.
	
//	Rail Points
	var factory1 = moi.command.createFactory( 'interpcurve' );
	var factory2 = moi.command.createFactory( 'interpcurve' );
	var factory3 = moi.command.createFactory( 'interpcurve' );
	
	for (var J = 1; J <= TIP; ++J)
	{
	//RDStype = 'Clockwise'
	//SDStype = 'BackSweep'
	if (RDStype == 'Clockwise' && SDStype == 'BackSweep' )
		{
			XP = R[J] * Math.cos( PHI[J] );
			YP = R[J] * Math.sin( PHI[J] );
			CXP = CHORD[J] * Math.cos(BETA[J] ) * Math.sin( PHI[J] + DELTA[J] );
			CYP = CHORD[J] * Math.cos(BETA[J] ) * Math.cos( PHI[J] + DELTA[J] );
			CZP = CHORD[J] * Math.sin(BETA[J] );
	// Create Leading Edge Rail curve here:
		var x = (XP + (0.25 * CXP)) * 1000;
		var y = (YP - (0.25 * CYP)) * 1000;
		var z = (0.25 * CZP) * 1000;
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory1.createInput( 'point' );
		factory1.setInput( factory1.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		
//	QtrChord:
		var x = XP * 1000;
		var y = YP * 1000;
		var z = 0.0;
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory2.createInput( 'point' );
		factory2.setInput( factory2.numInputs - 1, pt );		
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );
		
// Create Trailing Edge Rail curve here:
		x = (XP - (0.75 * CXP)) * 1000;
		y = (YP + (0.75 * CYP)) * 1000;
		z = (-0.75 * CZP) * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory3.createInput( 'point' );
		factory3.setInput( factory3.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );
		}
		
		else if (RDStype == 'CounterCW' && SDStype == 'BackSweep' )
		{
			XP = R[J] * Math.cos( PHI[J] )
			YP = R[J] * Math.sin( PHI[J] );
			CXP = CHORD[J] * Math.cos(BETA[J] ) * Math.sin( PHI[J] + DELTA[J] );
			CYP = CHORD[J] * Math.cos(BETA[J] ) * Math.cos( PHI[J] + DELTA[J] );
			CZP = CHORD[J] * Math.sin(BETA[J] );	

		var x = (XP + (0.25 * CXP)) * 1000;
		var y = (-YP + (0.25 * CYP)) * 1000;
		var z = 0.25 * CZP * 1000;
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory1.createInput( 'point' );
		factory1.setInput( factory1.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );
		
		x = XP * 1000;
		y = -YP * 1000;
		z =0.0;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory2.createInput( 'point' );
		factory2.setInput( factory2.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		

		x = (XP - (0.75 * CXP)) * 1000;
		y = (-YP - (0.75 * CYP)) * 1000;
		z = (-0.75 * CZP) * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory3.createInput( 'point' );
		factory3.setInput( factory3.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		
		}

		else if ((RDStype == 'Clockwise' && SDStype == 'ForwardSweep' ))
		{
			XP = R[J] * Math.cos( -PHI[J] );
			YP = R[J] * Math.sin( -PHI[J] );
			CXP = CHORD[J] * Math.cos(BETA[J] ) * Math.sin( -PHI[J] - DELTA[J] );
			CYP = CHORD[J] * Math.cos(BETA[J] ) * Math.cos( -PHI[J] - DELTA[J] );
			CZP = CHORD[J] * Math.sin(BETA[J] );	

		var x = (XP + (0.25 * CXP)) * 1000;
		var y = (YP - (0.25 * CYP)) * 1000;
		var z = 0.25 * CZP * 1000;
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory1.createInput( 'point' );
		factory1.setInput( factory1.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		

		x = XP * 1000;
		y = YP * 1000;
		z = 0.0 * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory2.createInput( 'point' );
		factory2.setInput( factory2.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		

		x = (XP - (0.75 * CXP)) * 1000;
		y = (YP + (0.75 * CYP)) * 1000;
		z = -0.75 * CZP * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory3.createInput( 'point' );
		factory3.setInput( factory3.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		
		}

		else if ((RDStype == 'CounterCW' && SDStype == 'ForwardSweep' ))
		{
			XP = R[J] * Math.cos( -PHI[J] );
			YP = R[J] * Math.sin( -PHI[J] );
			CXP = CHORD[J] * Math.cos(BETA[J] ) * Math.sin( -PHI[J] - DELTA[J] );
			CYP = CHORD[J] * Math.cos(BETA[J] ) * Math.cos( -PHI[J] - DELTA[J] );
			CZP = CHORD[J] * Math.sin(BETA[J] );	
//
		var x = (XP + (0.25 * CXP)) * 1000;
		var y = (-YP + (0.25 * CYP)) * 1000;
		var z = 0.25 * CZP * 1000;
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory1.createInput( 'point' );
		factory1.setInput( factory1.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		

		x = XP * 1000;
		y = -YP * 1000;
		z = 0.0 * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory2.createInput( 'point' );
		factory2.setInput( factory2.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		

		x = (XP - (0.75 * CXP)) * 1000;
		y = (-YP - (0.75 * CYP)) * 1000;
		z = -0.75 * CZP * 1000;		
		var pt = moi.vectorMath.createPoint( x, y, z );	
		factory3.createInput( 'point' );
		factory3.setInput( factory3.numInputs - 1, pt );
//	Save the control points.
		var pointfactory = moi.command.createFactory( 'point' );		
		pointfactory.setInput( 0, pt );
		ControlPointsfactories.push( pointfactory );		
		}
	}
//	Save each factory, (one curve each).
		LeadingEdgefactory.push( factory1 );
		QtrChordfactory.push( factory2 );		
		TrailingEdgefactory.push( factory3 );
	
//	Four factory(s) to update.
	for ( var k = 0; k < LeadingEdgefactory.length; ++k )
	{
		LeadingEdgefactory[k].update();
	}
	for ( var k = 0; k < QtrChordfactory.length; ++k )
	{
		QtrChordfactory[k].update();
	}
	for ( var k = 0; k < TrailingEdgefactory.length; ++k )
	{
		TrailingEdgefactory[k].update();
	}	
	for ( var k = 0; k < ControlPointsfactories.length; ++k )
	{
		ControlPointsfactories[k].update();
	}
	return airFoilProfile;	
//			alert("Script has reached the end")
}

function DoPropDesign()
{
//	The AirFoil is based upon the following:
//	Based upon NACA data, PROP_DESIGN_GEO 1/5/2012.
//	Array index starts at zero, and ends at 24.
//	There are 25 NACA points, which will form a control point type of curve.
//  The leading edge will be part of a conic ellipse.
//	the trailing edge will be a very small 3 point arc.

	var AFXTAB = new Array(25);
		AFXTAB = [ 0.50, 0.75, 1.250, 2.50, 5.0, 7.50, 10.0, 15.0, 20.0, 25.0, 
		30.0, 35.0, 40.0, 45.0, 50.0, 55.0, 60.0, 65.0, 70.0, 75.0, 80.0, 
		85.0, 90.0, 95.0, 99.982104290 ];
	var AFYTAB = new Array(25);
		AFYTAB = [ 0.690, 0.8370, 1.0680, 1.4630, 1.9650, 2.3850, 2.7360, 3.2920, 
		3.7140, 4.0340, 4.2660, 4.4200, 4.4950, 4.4860, 4.3790, 4.1740, 3.8810, 
		3.5190, 3.0990, 2.6310, 2.1270, 1.6020, 1.0750, 0.5470, 0.0198889890 ]

		
//	The next three arrays hold the three rail curves, based upon "through points."
//	The ControlPointsFactories array, permits viewing the control points.
	var LeadingEdgefactory = new Array();	
	var QtrChordfactory = new Array();	
	var TrailingEdgefactory = new Array();
	var ControlPointsfactories = new Array();
	
	var BETA = new Array();
	var CHORD = new Array();
	var DELTA = new Array();
	var DLP = new Array();
	var DPHI = new Array();
	var DR = new Array();
	var PHI = new Array();
	var R = new Array();
	var X = new Array();
	var Y = new Array();
	var DX = new Array();
	var DY = new Array();

	var ui = moi.ui.commandUI;  //Abbreviation
 	// Show the initial result.
	var airFoilProfile = Update( AFXTAB, AFYTAB, BETA, CHORD, DELTA, DLP, DPHI, DR, PHI, R, X, Y, DX, DY, ui.TIP.integerValue, ui.PITCH.numericValue, ui.RADIUS.numericValue, ui.ROCR.numericValue, ui.PHIMAX.numericValue, ui.CDS.value, ui.RDS.value, ui.SDS.value, LeadingEdgefactory, QtrChordfactory, TrailingEdgefactory, ControlPointsfactories, null );	

	// Wait for any changes to the UI or cancel or done.
	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			if ( airFoilProfile != null )
			{
				moi.geometryDatabase.removeObjects( airFoilProfile );		
			}
			Cancel( LeadingEdgefactory );
			Cancel ( QtrChordfactory );		
			Cancel ( TrailingEdgefactory );
//Note, the next line cannot be before removalObjects( airFoilProfile );			
			Cancel ( ControlPointsfactories );
			return; // Canceled.
		}

		if ( dialog.event == 'done' )
			break; // "Done" pushed.

		if ( dialog.event == 'TIP' || dialog.event == 'PITCH' || dialog.event == 'RADIUS' || dialog.event == 'ROCR' || dialog.event == 'PHIMAX' || dialog.event == 'CDS' || dialog.event == 'RDS' || dialog.event == 'SDS' )
		{
	airFoilProfile = Update( AFXTAB, AFYTAB, BETA, CHORD, DELTA, DLP, DPHI, DR, PHI, R, X, Y, DX, DY, ui.TIP.integerValue, ui.PITCH.numericValue, ui.RADIUS.numericValue, ui.ROCR.numericValue, ui.PHIMAX.numericValue, ui.CDS.value, ui.RDS.value, ui.SDS.value, LeadingEdgefactory, QtrChordfactory, TrailingEdgefactory, ControlPointsfactories, airFoilProfile );
		}
	}
// Tells MoI: Do Not remove geometry during cleanup.	
	Commit(LeadingEdgefactory );
	Commit(QtrChordfactory);	
	Commit(TrailingEdgefactory);
	Commit(ControlPointsfactories);
	
}

DoPropDesign();