#include "GetPoint.js"

var g_LineFactory;
var g_PointPicker;
var g_StartFrame;
var g_Output = null;

function CrossProduct( v1, v2 )
{
	return moi.vectorMath.createPoint( (v1.y * v2.z) - (v1.z * v2.y), (v1.z * v2.x) - (v1.x * v2.z), (v1.x * v2.y) - (v1.y * v2.x) );
}

function ExtractObjectFromList( list )
{
	if ( list.length != 1 )
		return null;

	return list.item(0);
}

function Join( objA, objB, objC, objD )
{
	var list = moi.geometryDatabase.createObjectList();
	list.addObject( objA );
	list.addObject( objB );
	list.addObject( objC );
	list.addObject( objD );

	var f = moi.command.createFactory( 'join' );
	f.setInput( 0, list );
	
	var res = f.calculate();
	f.cancel();

	return ExtractObjectFromList( res );
}

function CreateArc( frame, startpt )
{
	var f = moi.command.createFactory( 'arccenter' );
	f.setInput( 0, frame );
	f.setInput( 1, startpt );
	f.setInput( 3, 180 );

	var res = f.calculate();
	f.cancel();

	return ExtractObjectFromList( res );
}

function CreateLine( startpt, endpt )
{
	var f = moi.command.createFactory( 'line' );
	f.setInput( 0, startpt );
	f.setInput( 1, endpt );

	var res = f.calculate();
	f.cancel();

	return ExtractObjectFromList( res );
}

function CreatePoint( pt )
{
	var f = moi.command.createFactory( 'point' );
	f.setInput( 0, pt );

	var res = f.calculate();
	f.cancel();

	return ExtractObjectFromList( res );
}

function ClearOutput()
{
	if ( g_Output )
	{
		moi.geometryDatabase.removeObjects( g_Output );
		g_Output = null;
	}
}

function Update()
{
	ClearOutput();

	var line = ExtractObjectFromList( g_LineFactory.getCreatedObjects() );
	if ( !line )
		return;

	g_Output = moi.geometryDatabase.createObjectList();

	var zaxis = g_StartFrame.zaxis;
	var startpt = line.getStartPt();
	var endpt = line.getEndPt();

	var dir = moi.vectorMath.makeVector( startpt, endpt );
	var len = Math.sqrt( dir.x * dir.x + dir.y * dir.y + dir.z * dir.z );
	var unit_dir = moi.vectorMath.createPoint( dir.x / len, dir.y / len, dir.z / len );

	var radius = moi.ui.commandUI.radius.value;
	if ( radius <= 0.0 )
		return;

	var unit_perp = CrossProduct( zaxis, unit_dir );
	var perp = moi.vectorMath.createPoint( unit_perp.x, unit_perp.y, unit_perp.z );
	perp.scale( radius );

	var left_line_start = line.getStartPt();
	left_line_start.add( perp );

	var left_line_end = line.getEndPt();
	left_line_end.add( perp );

	perp.scale( -1.0 );
	var right_line_start = line.getStartPt();
	right_line_start.add( perp );

	var right_line_end = line.getEndPt();
	right_line_end.add( perp );

	var left_line = CreateLine( left_line_start, left_line_end );
	var right_line = CreateLine( right_line_start, right_line_end );

	var start_arc_frame = moi.vectorMath.createFrame( line.getStartPt(), perp, unit_dir );
	var start_arc = CreateArc( start_arc_frame, left_line_start );

	perp.scale( -1.0 );
	var end_arc_frame = moi.vectorMath.createFrame( line.getEndPt(), perp, unit_dir );
	var end_arc = CreateArc( end_arc_frame, left_line_end );

	var slot = Join( left_line, right_line, start_arc, end_arc );
	slot.setHitTest( false );
	g_Output.addObject( slot );
	moi.geometryDatabase.addObjects( g_Output );
}

function DoSlot()
{
	g_PointPicker = moi.ui.createPointPicker();
	if ( !GetPoint( g_PointPicker ) )
		return;

	g_StartFrame = g_PointPicker.ptFrame;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'StartPrompt' );
	moi.ui.showUI( 'EndPrompt' );
	moi.ui.endUIUpdate();

	g_LineFactory = moi.command.createFactory( 'line' );
	g_LineFactory.setInput( 0, g_PointPicker.pt );

	g_PointPicker.bindBasePt( g_LineFactory.getInput(0) );
	g_PointPicker.bindResultPt( g_LineFactory.getInput(1) );
	g_PointPicker.bindFuncPostUpdate( Update );
	moi.ui.bindUIToInput( 'bothsides', 'value', g_LineFactory.getInput(2) );

	g_PointPicker.restrictToPlane( g_StartFrame, true );

	while ( 1 )
	{
		if ( !g_PointPicker.waitForEvent() )
		{
			ClearOutput();
			break;
		}
			
		if ( g_PointPicker.event == 'finished' )
			break;

		if ( g_PointPicker.event == 'radius' || g_PointPicker.event == 'bothsides' )
			Update();
	}

	g_LineFactory.cancel();

	if ( g_Output && g_Output.length == 1 )
		g_Output.item(0).setHitTest( true );
}

DoSlot();
