// config: norepeat

#include "GetPoint.js"

function concat( list )
{
	for ( var i = 1; i < arguments.length; i++ )
	{
		var arglist = arguments[i];
	
		for ( var j = 0; j < arglist.length; j++ )
			list.addObject( arglist.item(j) );
	}

	return list;
}

function factory( factoryname )
{
	var f = moi.command.createFactory( factoryname );

	for ( var i = 1; i < arguments.length; i++ )
		f.setInput( i - 1, arguments[i] );

	var objects = f.calculate();
	f.cancel();

	return objects;
}

function UpdateGear( N, DP, cplane, obj )
{
	var gd = moi.geometryDatabase;

	if ( obj != null )
		gd.removeObjects( obj );

	if ( N == 0 || DP == 0 || !isFinite( DP ) )
		return null;

	var numPt = 5;					// Sample points per involute curve
	var PA = 20 * Math.PI / 180;			// Pressure Angle

	var PD = N / DP;				// Pitch Diameter
	var BD = PD * Math.cos( PA );			// Base Circle Diameter
	var A  = 1.0 / DP;				// Addendum
	var D  = 1.157 / DP;				// Dedendum
	var OD = PD + 2 * A;				// Outside Diameter
	var RD = PD - 2 * D;				// Root Diameter

	var alfa  = ( Math.PI / N ) / 2 - PA + Math.sqrt( Math.pow( PD / BD, 2 ) - 1.0 );
	var omega = alfa - Math.sqrt( Math.pow( OD / BD, 2 ) - 1.0  );

	var beta = RD > BD ? Math.sqrt( Math.pow( RD / BD, 2 ) - 1.0 ) : 0.0;
	
	--numPt;
	var step = ( alfa - beta - omega ) / numPt;

	var curvefactory = moi.command.createFactory( 'interpcurve' );

	var z = 0.0;

	for ( var i = 0; i <= numPt; i++ )
	{
		var delta = i * step + beta;

		var r = Math.sqrt( Math.pow( BD / 2, 2 ) * ( Math.pow( delta, 2 ) + 1 ) );
		var theta = alfa - delta + Math.atan( delta );

		var x = r * Math.cos( theta );
		var y = r * Math.sin( theta );

		var pt = cplane.evaluate( x, y, z );

		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );
	}

	var arcPt1 = cplane.evaluate( x, -y, z );
	var arcPt2 = pt;

	var arcPt3 = curvefactory.getInput(0).getValue();
	var arcPt4;

	var curve = curvefactory.calculate();
	curvefactory.cancel();

	var line = gd.createObjectList(); // add hoc...
	if ( RD < BD )
	{
		var linePt = arcPt3;
		x = RD / 2 * Math.cos( alfa );
		y = RD / 2 * Math.sin( alfa );
		arcPt3 = cplane.evaluate( x, y, z );
		line = factory( 'line', linePt, arcPt3 );
	}

	var curves = factory( 'mirror', concat( curve, line ), cplane, cplane.evaluate( 1.0, 0.0, 0.0 ) );

	var arc = factory( 'arccenter', cplane, arcPt1, arcPt2 );

	var theta0 = alfa - beta + Math.atan( beta );
	arcPt4 = 2 * ( Math.PI - N * theta0 ) / N; // angular measure
	var r_arc = factory( 'arccenter', cplane, arcPt3, -1, arcPt4 * 180/Math.PI );

	var tooth = concat( curves, arc, curve, line, r_arc );

	var clones = factory( 'arraycircular', tooth, cplane, N, 360.0 );

	var profile = factory( 'join', concat( tooth, clones ) );

	gd.addObjects( profile );
	profile.lockSelection();

	return profile;
}


function DoGear()
{
	var pointpicker = moi.ui.createPointPicker();
	if ( !GetPoint( pointpicker ) )
		return;

	var cplane = pointpicker.ptframe;

	var ui = moi.ui.commandUI;

	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'PickPrompt' );
	moi.ui.hideUI( 'CancelContainer' );
	moi.ui.showUI( 'OptionsPrompt' );

	var units = moi.geometryDatabase.Units;
	var T; // transpose
	if ( units == 'Inches' || units == 'Feet' || units == 'Miles' )
	{
		T = 1.0;
		ui.DiamPitch.style.display = 'inline';
	}
	else
	{
		T = -1.0;
		ui.Module.style.display = 'inline';
	}

	moi.ui.showUI( 'GearOptions' );
	moi.ui.showUI( 'DoneCancelContainer' );
	moi.ui.endUIUpdate();

	var N  = ui.N.integerValue;				// Number of Teeth
	var DP = Math.abs( Math.pow( ui.DP.numericValue, T ) );	// Diametral Pitch

	// Show the initial result.
	var profile = UpdateGear( N, DP, cplane, null );

	// Wait for any changes to the UI or cancel or done.
	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			if ( profile != null )
				moi.geometryDatabase.removeObjects( profile );
			return; // Canceled.
		}

		if ( dialog.event == 'done' )
			break; // "Done" pushed.

		if ( dialog.event == 'N' )
		{
			// Number of teeth control changed.
			N = ui.N.integerValue;
			profile = UpdateGear( N, DP, cplane, profile );
		}
		else if ( dialog.event == 'DP' )
		{
			// Diametral pitch control changed.
			DP = Math.abs( Math.pow( ui.DP.numericValue, T ) );
			profile = UpdateGear( N, DP, cplane, profile );
		}
	}

	// Draw pitch diameter circle.
	var circlefactory = moi.command.createFactory( 'circle' );
	circlefactory.setInput( 0, false );
	circlefactory.setInput( 1, cplane );
	circlefactory.setInput( 3, N / DP );
	circlefactory.commit();
}

DoGear();
