// Wire v.1.0 - Max Smirnov. 2017
// Updated version v.1.0.1 - Marco (mkdm). Feb 2017.
// This version allows the selection of a source profile for the Sweep command.
// If no source profile is selected then a factory Circle is used and the Diameter Input UI is displayed.

#include "GetObjects.js"

function Wire()
{
	moi.selection.setFilter( 'Types', 'Curves', true );
	moi.selection.setFilter( 'Types', 'Edges', true );
	var objectpicker = moi.ui.createObjectPicker();
	objectpicker.allowCurves();
	objectpicker.allowStandaloneCurves();
	objectpicker.allowEdgeCurves();
	
	if ( !GetObjects( objectpicker, true ) )
		return;
	
	moi.selection.clearSelectionFilters();
	
	var sweepProfile = objectpicker.objects;
	
	sweepProfile.sortBySelectionOrder();
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt0' );
	moi.ui.showUI( 'SelectPrompt1' );
	moi.ui.endUIUpdate();	
	
	var picker = moi.ui.createOrientationPicker();
	picker.noDefaultFrame = true;
	
	while ( true )
	{
		if ( !picker.waitForEvent() ) return;
		if ( picker.event === 'finished' || picker.event === 'done' )
		{
			moi.ui.commandUI.frame1 = moi.VectorMath.createFrame(picker.frame.origin, picker.frame.xaxis, picker.frame.yaxis );
			break;
		}
	}

	picker.reset();
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt1' );
	moi.ui.showUI( 'SelectPrompt2' );
	moi.ui.endUIUpdate();
	
	while ( true )
	{
		if ( !picker.waitForEvent() ) return;
		if ( picker.event === 'finished' || picker.event === 'done' ) break;
	}
	
	moi.ui.commandUI.frame2 = moi.VectorMath.createFrame(picker.frame.origin, picker.frame.xaxis, picker.frame.yaxis );
	picker.reset();
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt2' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'Options' );
	moi.ui.endUIUpdate();
	
	moi.ui.commandUI.init(sweepProfile);
	
	while ( true )
	{
		moi.ui.commandDialog.waitForEvent();
		var e  = moi.ui.commandDialog.event;
		if ( e === 'cancel' ) { moi.ui.commandUI.cancel(); break; }
		if ( e === 'done' ) { moi.ui.commandUI.done(); break; }
		if ( e === 'keepCurves' ) { moi.ui.commandUI.ld += 1 }
	}
}

Wire();